##############################################################################
#
#----------------------------------------------------------------------------------------------------------------------------------------
#   elementpatcher.py
#
#   Classes to replace elements in ZMI
#
#   Copyright (c) 2004 Zope Japan KK.(Atsushi Shibata)
#   All Rights Reserved.
#----------------------------------------------------------------------------------------------------------------------------------------
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

#Class to replace ZMI elements

#import from python modules
from string import find
import re

__doc__="""elementpatcher.py : Class to replace ZMI elements'
$Id: elementpatcher.py,v 1.5 2004/12/24 08:13:14 ats_shib Exp $"""

__version__='$Revision: 1.5 $'[11:-2]

#
#constants
#

#Patch template for translate DTML
dtml_tmpl = '<dtml-translate domain=%s>%s</dtml-translate>'

#Patch template for translate PZT button
button_pt_tmpl = '%s" i18n:attributes="value" i18n:domain="%s'

#Patch template for ZMI labels,description
tmpl_pt = '<span i18n:translate="" i18n:domain="%s">%s</span>'

#regex flags
flg = re.M | re.I | re.S

#Domains for translations
zmi_domain = "zmi"
button_domain = "zmi-buttons"
tab_domain = "zmi-tabs"


def escape_keywords(s):
    """Escape keywords for regex special characters."""
    #Escape charactor list for regex patterns
    esc_list = ['?','*','.','(',')','[',']']
    for item in esc_list:
        s = s.replace(item,'\\' + item)
    return s

def isStringType(s):
    try:
        s = s + ''
        return True
    except:
        return False

class elementpatcher:
    """Class to patch ZMI form elements"""

    def __init__(self,src,patchers=[]):
        """ Initialize method for elementpatcher.
            src is ZMI form srouce(non-rendered).
            patchers are list of patcher instances."""
        #list for converted source
        self.c_src = [src]
        #Sort patchers
        patchers.sort(lambda x, y: len(x)-len(y))
        patchers.reverse()
        #Patchers
        self.patchers = patchers

    def replace(self):
        """ Replace all elements using patcher """

        #Loop over patcher list
        for pitem in self.patchers:
            # Loop over c_src
            # When element for translation find,split string and
            # Replace element to patcher instance.
            post_src = []
            for src_part in self.c_src:
                if isStringType(src_part):
                    element_area = pitem.findall(src_part)
                    if element_area:
                        # Elements found...
                        offset = 0
                        #split to parts.
                        for a_area in element_area:
                            post_src.append(src_part[offset:a_area[0]])
                            post_src.append(pitem)
                            offset = a_area[1]
                        post_src.append(src_part[offset:])
                    else:
                        # Elements not found. So simply append source_part
                        post_src.append(src_part)
                else:
                    post_src.append(src_part)
            self.c_src = post_src

    def render(self,isZPT=False):
        renderd = ''
        for src_part in self.c_src:
            if isStringType(src_part):
                renderd += src_part
            else:
                if not isZPT:
                    #render for DTML
                    renderd += src_part.get_replacestring_dt()
                else:
                    #render for ZPT
                    renderd += src_part.get_replacestring_pt()
        return renderd


class desc:
    """Patcher class for desciptions"""

    _type = 'Description patcher'

    def __init__(self,patchstring,msgid=''):
        """ Initialize method """
        self.s = patchstring
        self.msgid = msgid

    def __repr__(self):
        return "<%s: %s >" % (self._type,self.s)

    def __len__(self):
        return len(self.s)

    def find(self,src):
        """ Find area for patchstring """
        st = find(src,self.s)
        if st != -1:
            return ( st,st+len(self.s) )

        return None

    def findall(self,src):
        """ Find out all elements in src, and return list of item position """
        #Initialize offset
        offset = 0
        pos_t = []
        while True:
            pos = self.find(src[offset:])
            if not pos:
                break
            pos_t.append((pos[0]+offset,pos[1]+offset))
            offset = offset + pos[1]
        return pos_t

    def get_replacestring_dt(self):
        """ Return replace string for DTML """
        return dtml_tmpl % (zmi_domain,self.msgid)

    def get_replacestring_pt(self):
        """ Return replace string for ZPT """
        return tmpl_pt % (zmi_domain,self.msgid)


class label(desc):
    """ZMI Patcher class for form label"""

    _type = 'Label patcher'

    def __init__(self,patchstring,msgid=''):
        """ Initialize method """

        #call super class initialize method
        desc.__init__(self,patchstring,msgid)
        self.pat = None
        if len(self.s.split(' ')) <= 2:
            #target label has single word...
            #so use re.search and replace
            self.pat = re.compile('([>\s]+)(%s)' % escape_keywords(self.s))

    def find(self,src):
        """ Find area for label patchstring """
        if self.pat:
            #target label has multi words
            pm = self.pat.search(src)
            if pm:
                return( pm.start() + len(pm.groups()[0]) ,\
                     pm.start() + len(pm.groups()[0]) + len(pm.groups()[1]) )
        else:
            st = find(src,self.s)
            if st != -1:
                return ( st,st+len(self.s) )

        return None


class button(desc):
    """ZMI Patcher class for form label"""

    _type = 'Button patcher'

    def __init__(self,patchstring,msgid=''):
        """ Initialize method
            """
        #call super class initialize method
        desc.__init__(self,patchstring,msgid)
        self.pat = re.compile('("\s*)(' + \
                     escape_keywords(patchstring) + ')(\s*["&])')

    def find(self,src):
        """ Find area for label patchstring """
        if self.pat:
            #target label has multi words
            pm = self.pat.search(src)
            if pm:
                tail = 0
                #if pm.groups()[2].find('&') == -1:
                #    tail = len(pm.groups()[2])
                return( pm.start()+len(pm.groups()[0]),\
                         pm.start()+len(pm.groups()[0]) + len(pm.groups()[1]) \
                          + tail )

        return None

    def get_replacestring_pt(self):
        """ Return replace string for ZPT """
        return button_pt_tmpl % (self.msgid,zmi_domain)


class replace(desc):
    """ZMI Patcher class,simply replace string"""

    _type = 'Simple replacement'

    def __init__(self,patchstring,msgid):
        """ Initialize method
            patchstring is for source string to replace,
            msgid is for destination string to replace"""
        #call super class initialize method
        desc.__init__(self,patchstring,msgid)

    def get_replacestring_dt(self):
        """ Return replace string for DTML """
        return self.msgid

    def get_replacestring_pt(self):
        """ Return replace string for ZPT """
        return self.msgid



if __name__ == '__main__':
    src = open('test/main.dtml').read()
    pts = []
    desc_s = """You may edit the source for this document using the form below. You 
may also upload the source for this document from a local file. Click
the <em>browse</em> button to select a local file to upload."""
    pts.append(desc(desc_s,'dtml_desc1'))
    pts.append(button('Taller','Taller'))
    pts.append(button('Wider','Wider'))
    pts.append(label('Title','Title'))
    pts.append(label('File','File'))
    pts.append(label('Type','Type'))

    for item in pts:
        t = item.findall(src)
        print item.s,"::"
        for pos in t:
            print pos
            print src[max(pos[0]-100,0):min(pos[1]+100,len(src))]
        print "=" * 40

    print "*" * 40
    patcher = elementpatcher(src,pts)
    patcher.replace()
    #print patcher.c_src
    print patcher.render()
